@@ -1,4 +1,4 @@
-static char rcsid[]= "$Id: xlbiff.c,v 1.64 1992/09/24 15:04:51 PDT esm Exp $";
+static char rcsid[]= "$Id: xlbiff.c,v 1.75 2003/11/08 23:40:18 esm Exp $";
 /*\
 |* xlbiff  --  X Literate Biff
 |*
@@ -10,20 +10,22 @@ static char rcsid[]= "$Id: xlbiff.c,v 1.64 1992/09/24 15:04:51 PDT esm Exp $";
 |*	Xlbiff is modeled after xconsole; it remains invisible at non-
 |*	useful times, eg, when no mail is present.  See README for details.
 |*
-|*	Author:		Eduardo Santiago Munoz,  esm@auspex.com
+|*	Author:		Eduardo Santiago, esm@pobox.com
 |* 	Created:	20 August 1991
-|*	Last Updated:	24 September 1992
+|*	Last Updated:	 8 November 2003
 |*
-|*	Copyright 1991, 1992 Eduardo Santiago Munoz
-|*	Portions copyright 1991 Digital Equipment Corporation
+|*    Copyright 1994 Eduardo Santiago
 |*
-|*	Permission to use, copy, modify, distribute, and sell this software
-|*	and its documentation for any purpose is hereby granted without
-|*	fee, provided that the above copyright notice appear in all copies
-|*	and that both that copyright notice and this permission notice
-|*	appear in supporting documentation.  The author makes no repre-
-|*	sentation about the suitability of this software for any purpose.
-|*	It is provided "as is" without express or implied warranty.
+|*    The X Consortium, and any party obtaining a copy of these files from
+|*    the X Consortium, directly or indirectly, is granted, free of charge, a
+|*    full and unrestricted irrevocable, world-wide, paid up, royalty-free,
+|*    nonexclusive right and license to deal in this software and
+|*    documentation files (the "Software"), including without limitation the
+|*    rights to use, copy, modify, merge, publish, distribute, sublicense,
+|*    and/or sell copies of the Software, and to permit persons who receive
+|*    copies from any such party to do so.  This license includes without
+|*    limitation a license to do the foregoing actions under any patents of
+|*    the party supplying this software to the X Consortium.
 |*
 \*/
 
@@ -56,7 +58,7 @@ static char rcsid[]= "$Id: xlbiff.c,v 1.64 1992/09/24 15:04:51 PDT esm Exp $";
 typedef int             waitType;
 # define INTWAITTYPE
 #else /* ! X_NOT_POSIX */
-#ifdef SYSV
+#if	defined(SYSV) || defined(SVR4)
 # define waitCode(w)    (((w) >> 8) & 0x7f)
 # define waitSig(w)     ((w) & 0xff)
 typedef int             waitType;
@@ -105,6 +107,7 @@ void	Shrink(Widget, caddr_t, XEvent*, Boolean*);
 void	handler(XtPointer,XtIntervalId*);
 void	initStaticData(int*,int*,int*);
 void	Exit(Widget, XEvent*, String*, Cardinal*);
+void	Mailer(Widget, XEvent*, String*, Cardinal*);
 void	lbiffUnrealize(), lbiffRealize(char*);
 void	getDimensions(char*,Dimension*,Dimension*);
 void	toggle_key_led(int);
@@ -115,6 +118,7 @@ void	Shrink();
 void	handler();
 void	initStaticData();
 void	Exit();
+void	Mailer();
 void	lbiffUnrealize(), lbiffRealize();
 void	getDimensions();
 void	toggle_key_led();
@@ -143,6 +147,7 @@ typedef struct {
     char	*file;			/* file to monitor size of 	*/
     char	*checkCmd;		/* command to run for check     */
     char	*cmd;			/* command to run for output	*/
+    char	*mailerCmd;		/* command to read mail		*/
     int		update;			/* update interval, in seconds	*/
     int		fade;			/* popdown interval, in seconds */
     int		columns;		/* number of columns across	*/
@@ -168,6 +173,8 @@ static XtResource	xlbiff_resources[] = {
 	offset(checkCmd), XtRString, NULL},
     { "scanCommand", "ScanCommand", XtRString, sizeof(String),
 	offset(cmd), XtRString, "scan -file %s -width %d" },
+    { "mailerCommand", "MailerCommand", XtRString, sizeof(String),
+	offset(mailerCmd), XtRString, NULL },
     { "update", "Interval", XtRInt, sizeof(int),
 	offset(update), XtRImmediate, (XtPointer)15},
     { "fade", "Fade", XtRInt, sizeof(int),
@@ -208,7 +215,9 @@ static XrmOptionDescRec	optionDescList[] = {
     { "-led",         ".led",         XrmoptionSepArg,	(caddr_t) NULL},
     { "-ledPopdown",  ".ledPopdown",  XrmoptionNoArg,	(caddr_t) "true"},
     { "+ledPopdown",  ".ledPopdown",  XrmoptionNoArg,	(caddr_t) "false"},
+    { "-sound",	      ".sound",       XrmoptionSepArg,  (caddr_t) NULL},
     { "-scanCommand", ".scanCommand", XrmoptionSepArg,	(caddr_t) NULL},
+    { "-mailerCommand",".mailerCommand",XrmoptionSepArg,(caddr_t) NULL},
     { "-checkCommand",".checkCommand",XrmoptionSepArg,  (caddr_t) NULL}
 };
 
@@ -220,6 +229,7 @@ static char *fallback_resources[] = {
 
 static XtActionsRec lbiff_actions[] = {
     {"exit",Exit},
+    {"mailer",Mailer},
     {"popdown",Popdown}
 };
 
@@ -262,7 +272,7 @@ main(argc, argv)
     ** Check command line arguments
     */
     if (argc > 1) {
-	if (!strncmp(argv[1],"-v",2)) {
+	if (!strncmp(argv[1],"-version",strlen(argv[1]))) {
 	    fprintf(stderr,
 #if	TESTLEVEL != 0
 		    "%s version %d.%d.%d\n",
@@ -308,15 +318,27 @@ main(argc, argv)
     }
     DP(("file= %s\n",lbiff_data.file));
 
+
+    /*
+    ** Fix DISPLAY environment variable, might be needed by subprocesses
+    */
+    {
+      char *envstr = (char*)malloc(strlen("DISPLAY=") + 1
+			    + strlen(XDisplayString(XtDisplay(topLevel))));
+
+      sprintf(envstr, "DISPLAY=%s", XDisplayString(XtDisplay(topLevel)));
+      putenv(envstr);
+    }
+
     textBox = XtVaCreateManagedWidget("text",
 				      commandWidgetClass,
 				      topLevel,
 				      NULL);
 
-    XtAddCallback(textBox,XtNcallback, Popdown, textBox);
-    XtAppAddActions(app_context,lbiff_actions,XtNumber(lbiff_actions));
-    XtAddEventHandler(topLevel,StructureNotifyMask,False,
-		      (XtEventHandler)Shrink,(caddr_t)NULL);
+    XtAddCallback(textBox, XtNcallback, Popdown, textBox);
+    XtAppAddActions(app_context, lbiff_actions, XtNumber(lbiff_actions));
+    XtAddEventHandler(topLevel, StructureNotifyMask, False,
+		      (XtEventHandler)Shrink, (caddr_t)NULL);
 
     XtOverrideTranslations(topLevel,
 	XtParseTranslationTable ("<Message>WM_PROTOCOLS: exit()"));
@@ -349,6 +371,7 @@ Usage()
 {
     static char *help_message[] = {
 "where options include:",
+"    -version                           display xlbiff version number",
 "    -display host:dpy                  X server to contact",
 "    -geometry +x+y                     x,y coords of window",
 "    -rows height                       height of window, in lines",
@@ -364,6 +387,7 @@ Usage()
 "    -ledPopdown                        turn off LED when popped down",
 "    -scanCommand command               command to interpret and display",
 "    -checkCommand command              command used to check for change",
+"    -mailerCommand command             command used to read mail",
 NULL};
     char **s;
 
@@ -389,7 +413,7 @@ String *params;
 Cardinal *num_params;
 #endif
 {
-    DP(("++exit()\n"));
+    DP(("++Exit()\n"));
 
     if (event->type == ClientMessage) {
 	if (event->xclient.data.l[0] != wm_delete_window) {
@@ -408,6 +432,31 @@ Cardinal *num_params;
 }
 
 
+/************\
+|*  Mailer  *|  called via callback, starts a mailer
+\************/
+void
+#ifdef	FUNCPROTO
+Mailer(Widget w, XEvent *event, String *params, Cardinal *num_params)
+#else
+Mailer(w, event, params, num_params)
+Widget w;
+XEvent *event;
+String *params;
+Cardinal *num_params;
+#endif
+{
+    DP(("++Mailer()\n"));
+
+    if (lbiff_data.mailerCmd != NULL && lbiff_data.mailerCmd[0] != '\0') {
+	Popdown();
+	system(lbiff_data.mailerCmd);
+	Popup();
+	checksize();
+    }
+}
+
+
 /***************\
 |*  checksize  *|  checks mail file to see if new mail is present
 |***************
@@ -429,27 +478,42 @@ checksize()
     DP(("++checksize()..."));
 
     /*
-    ** Do the stat to get the mail file size.  If it fails for any reason,
-    ** ignore the failure and assume the file is size 0.  Failures I can
-    ** think of are that should be ignored are:
-    **
-    **		+ nonexistent file.  Some Berkeley-style mailers delete
-    **		  the spool file when they're done with it.
-    **		+ NFS stale filehandle.  Yuk.  This one happens if
-    ** 		  your mail spool file is on an NFS-mounted directory
-    **		  _and_ your update interval is too low _and_ you use
-    **		  a Berkeleyish mailer.  Yuk.
-    **
-    ** Doubtless there are errors we should complain about, but this 
-    ** would get too ugly.
+    ** If user has specified a command to use to check the file, invoke
+    ** it with lbiff_data.file and "previous" as arguments, where "previous"
+    ** is the output of the script the last time it was run (or zero,
+    ** the first time we call it).  This is useful as a way of keeping
+    ** state for the checkCommand; in this manner it knows, if the
+    ** spool file size is nonzero, whether it has grown since the last
+    ** time we called it.
     */
-    if (lbiff_data.checkCmd != NULL) {
-	waitType status;
+    if (lbiff_data.checkCmd != NULL && lbiff_data.checkCmd[0] != '\0') {
+	FILE        *p;
+	waitType     status;
+	char	     outbuf[80];
+	static char *cmd_buf;
+	static int   previous;
+
+	if (cmd_buf == NULL) {
+	    cmd_buf = (char*)malloc(strlen(lbiff_data.checkCmd) +
+				    strlen(lbiff_data.file) + 10);
+	    if (cmd_buf == NULL)
+	      ErrExit(True,"scan command buffer malloc()");
+	}
+
+	sprintf(cmd_buf, lbiff_data.checkCmd, lbiff_data.file, previous);
+	DP(("++checkCommand= %s\n",cmd_buf));
+
+	if ((p= popen(cmd_buf,"r")) == NULL)
+	  ErrExit(True,"popen(checkCommand)");
+	if (fread(outbuf,1,sizeof outbuf,p) < 0)
+	  ErrExit(True,"fread(checkCommand)");
+	previous = atol(outbuf);
+	DP(("checkCommand returns %d\n",previous));
 
 #ifdef	INTWAITTYPE
-	status = 	  system(lbiff_data.checkCmd);
+	status = 	  pclose(p);
 #else
-	status.w_status = system(lbiff_data.checkCmd);
+	status.w_status = pclose(p);
 #endif
 	switch (waitCode(status)) {
 	case 0:					/* 0: new data */
@@ -462,6 +526,21 @@ checksize()
 	    mailstat.st_size = mailsize;
 	}
     } else {	/* no checkCmd, just stat the mailfile and check size */
+	/*
+	 ** Do the stat to get the mail file size.  If it fails for any reason,
+	 ** ignore the failure and assume the file is size 0.  Failures I can
+	 ** think of are that should be ignored are:
+	 **
+	 **		+ nonexistent file.  Some Berkeley-style mailers delete
+	 **		  the spool file when they're done with it.
+	 **		+ NFS stale filehandle.  Yuk.  This one happens if
+	 ** 		  your mail spool file is on an NFS-mounted directory
+	 **		  _and_ your update interval is too low _and_ you use
+	 **		  a Berkeleyish mailer.  Yuk.
+	 **
+	 ** Doubtless there are errors we should complain about, but this 
+	 ** would get too ugly.
+	 */
 	if (stat(lbiff_data.file,&mailstat) != 0) {
 	    DP(("stat() failed, errno=%d.  Assuming filesize=0!\n",errno));
 	    mailstat.st_size = 0;
@@ -553,7 +632,7 @@ doScan()
     static char	*cmd_buf;
     static char *buf = NULL;
     static int	bufsize;
-    static char scan_fail_msg[] = "\n---->>>>scanCommand failed<<<<<----\n";
+    static char scan_fail_msg[] = "\n---->>>> scanCommand failed <<<<<----\n";
     FILE 	*p;
     size_t	size;
     waitType	status;
@@ -704,8 +783,11 @@ Popdown()
     struct timezone tzp;
 
     DP(("++Popdown()\n"));
-    if (visible)
-      XtPopdown(topLevel);
+
+    if (visible) {
+	XtPopdown(topLevel);
+	XSync(XtDisplay(topLevel), False);
+    }
     visible = False;
 
     /*
@@ -720,6 +802,7 @@ Popdown()
       toggle_key_led(False);
 }
 
+
 void
 Popup()
 {
@@ -735,8 +818,10 @@ Popup()
       ErrExit(True,"gettimeofday() in Popup()");
     popup_time = tp.tv_sec;
 
-    if (hasdata && !visible)
-      XtPopup(topLevel, XtGrabNone);
+    if (hasdata && !visible) {
+	XtPopup(topLevel, XtGrabNone);
+	XSync(XtDisplay(topLevel), False);
+    }
     visible = True;
 }
 
@@ -810,8 +895,15 @@ lbiffRealize( s )
 
 
     if (lbiff_data.sound[0] == '\0') {
-	XBell(XtDisplay(topLevel),lbiff_data.volume - 100);
-	DP(("---sound= %s\n","XBell default"));
+	/*
+	** No, the following is not a typo, nor is it redundant code.
+	** Apparently there is one X terminal that beeps whenever XBell()
+	** is called, even with volume zero.
+	*/
+	if (lbiff_data.volume > 0) {
+	    XBell(XtDisplay(topLevel),lbiff_data.volume - 100);
+	    DP(("---sound= %s\n","XBell default"));
+	}
     }
     else {
 	static char	*sound_buf;
